(* $Id: libcl.ml,v 1.18 2002/02/17 21:09:31 ddr Exp $ *)
(* Copyright (c) 2002 INRIA *)

module Hashtbl =
  struct
    type ('a, 'b) t = ('a, 'b) hashtbl__t;;
    let new = hashtbl__create;;
    let add = hashtbl__add;;
    let find = hashtbl__find;;
  end
;;

module Queue =
  struct
    exception Empty = queue__Empty;;
    let new = queue__create;;
    let add = queue__add;;
    let take = queue__take;;
  end
;;

module Stack =
  struct
    exception Empty;;
    type 'a t = { mutable c : 'a list };;
    let new () = {c = []};;
    let clear s = s.c <- [];;
    let push x s = s.c <- x :: s.c;;
    let pop s =
      match s.c with
        hd :: tl -> s.c <- tl; hd
      | [] -> raise Empty
    ;;
    let length s = list__length s.c;;
    let iter f s = list__iter f s.c;;
  end
;;

module Stream =
  struct
    exception Parse_failure;;
    exception Parse_error;;
    type 'a t = 'a stream__t;;
    type 'a stream = 'a stream__t;;
    let do_stream = stream__iter;;
    let stream_check f strm =
      match stream__peek strm with
        Some t when f t -> stream__junk strm; t
      | _ -> raise Parse_failure
    ;;
    let stream_from f =
      stream__from
        (fun _ ->
           try Some (f ()) with
             stream__Failure | End_of_file -> None)
    ;;
    let stream_junk = stream__junk;;
    let stream_next = stream__next;;
    let stream_peek = stream__peek;;
    let stream_of_channel = stream__of_channel;;
    let stream_of_string = stream__of_string;;
    let icons = stream__icons;;
    let ising = stream__ising;;
    let lapp = stream__lapp;;
    let lcons = stream__lcons;;
    let lsing = stream__lsing;;
    let slazy = stream__slazy;;
  end
;;

module String =
  struct
    let blit_string = string__blit;;
    let create = string__create;;
    let length = string__length;;
    let make = string__make;;
    let nth_char = string__get;;
    let set_nth_char = string__set;;
    let sub = string__sub;;
  end
;;

module Sys =
  struct
    exception Sys_error = Sys_error;;
    exception Break = sys__Break;;
    type open_flag =
        O_RDONLY | O_WRONLY | O_RDWR | O_APPEND | O_CREAT
      | O_TRUNC | O_EXCL | O_BINARY | O_TEXT
    ;;
    let catch_break = sys__catch_break;;
    let command_line = sys__argv;;
    let interactive = !(sys__interactive);;
    let max_string_length = sys__max_string_length;;
    let remove = sys__remove;;
  end
;;

exception Parse_failure = stream__Parse_failure;;
exception Parse_error = stream__Parse_error;;

type 'a stream = 'a stream__stream;;
type 'a vect = 'a array;;

let add_int = prefix +;;
let assoc = list__assoc;;
let blit_string = string__blit_string;;
let blit_vect = array__blit;;
let combine (l1, l2) = list__combine l1 l2;;
let create_string = string__create;;
let copy_vect = array__copy;;
let do_list = list__iter;;
let do_list2 = list__iter2;;
let do_stream = stream__do_stream;;
let exists = list__exists;;
let for_all = list__for_all;;
let format_float form (floa : float) : string = printf__sprintf form floa;;
let index a =
  let rec index_rec i =
    function
      [] -> raise Not_found
    | b :: l -> if a = b then i else index_rec (succ i) l
  in
  index_rec 0
;;
let it_list = list__fold_left;;
let it_list2 = list__fold_left2;;
let list_it = list__fold_right;;
let list_length = list__length;;
let list_of_vect = array__to_list;;
let make_string = string__make;;
let make_vect = array__make;;
let map = list__map;;
let mem = list__mem;;
let mem_assoc = list__mem_assoc;;
let nth_char = string__nth_char;;
let rev = list__rev;;
let set_nth_char = string__set_nth_char;;
let std_in = stdin;;
let std_out = stdout;;
let stream_check = stream__stream_check;;
let stream_from = stream__stream_from;;
let stream_junk = stream__stream_junk;;
let stream_next = stream__stream_next;;
let stream_of_channel = stream__stream_of_channel;;
let stream_of_string = stream__stream_of_string;;
let stream_peek = stream__stream_peek;;
let string_length = string__length;;
let sub_string = string__sub;;
let sub_vect = array__sub;;
let subtract f =
  function
    [] -> f
  | e ->
      let rec subtract_e =
        function
          [] -> []
        | elem :: l ->
            if mem elem e then subtract_e l else elem :: subtract_e l
      in
      subtract_e f
;;
let union l1 l2 =
  let rec union_rec =
    function
      [] -> l2
    | a :: l -> if list__mem a l2 then union_rec l else a :: union_rec l
  in
  union_rec l1
;;
let vect_length = array__length;;
